//===================================================================
//                                                                  =
// Author:    Martin Berriman                                       =
// Copyright: Innovatronics Ltd 2006                                =
//                                                                  =
//===================================================================

#ifndef _CALIBRATEC
#define _CALIBRATEC

// HEADERS ==========================================================
#include "calibrate.h"
// END HEADERS ======================================================

// C FILES ==========================================================
// END C FILES ======================================================

// MACROS =========================================================
// END MACROS =====================================================

// GLOBALS ==========================================================
unsigned int16 new_cal_offset[2];
unsigned int32 new_cal_slope[2];
  signed int16 slope_errors[2][NUM_SLOPE_CAL_POINTS];
  signed int16 slope_points[NUM_SLOPE_CAL_POINTS] = {1800, 900, 1350, 450};
// END GLOBALS ======================================================

//----------------------------------------------------------------------------------
void show_measured_value(void)
//----------------------------------------------------------------------------------
{
	read_dual_pressures(&pressure[0], &pressure[1]);
	load_string_test_info(lcdStr, st_test_single_pressure);
	lcd_printf(1, 4, lcdStr);
}

//----------------------------------------------------------------------------------
void calibrate(void)
/*
When determining the slope correction, 4 measurements points are used to
calculate 4 slope correction factors.  These measurements will be spread across
the measurement range (probably 450Pa, 900Pa, 1350Pa and 1800Pa). Once these four
corrections factors are collected, they will be averaged to produce a final slope
correction factor.

The idea behind this is to try and reduce the effects of operator error during
calibration; hopefully the operator errors will not all occur in the same direction.
In order to try and ensure this, the measurements will be made in the following
order (1800Pa, 900Pa, 1350Pa and 450Pa).
*/
//----------------------------------------------------------------------------------
{
	bool abort;
	int8 key_pressed;
	int8 old_units;
	int8 i;

	test_state[0] = state_idle;
	test_state[1] = state_idle;

	abort = false;
	key_pressed = 0;
	reset_solenoids(0);
	reset_solenoids(1);
	testing = false;
	calibrating = true;
	old_units = units;

	cal_offset[0] = 0;			//default offset of 0
	cal_offset[1] = 0;			//default offset of 0
	cal_slope[0] = 100000;		//default slope of 1
	cal_slope[1] = 100000;		//default slope of 1
	new_cal_offset[0] = 0;
	new_cal_offset[1] = 0;
	new_cal_slope[0] = 100000;
	new_cal_slope[1] = 100000;

	units = st_units_pa;	//we do calibrations in Pa

	//Tell the user to set min pressure
	load_string(st_load_cal_info, st_calibrate_mode, lcdStr);
	lcd_printf(1, 2, lcdStr);
	load_string(st_load_cal_info, st_calibrate_min, lcdStr);
	lcd_printf(1, 3, lcdStr);
	load_string(st_load_cal_info, st_calibrate_start, lcdStr);
	lcd_printf(1, 4, lcdStr);

	//Wait for them to press - no way out of this apart from power off
	key_pressed = 0;
	timer_restart();

	while ((key_pressed == 0) && !abort)
	{
		abort = calibrate_idle(&key_pressed);
		//show_measured_value();
	}

	if (!abort)
	{
		load_string(st_load_cal_info, st_calibrate_wait, lcdStr);
		lcd_printf(1, 4, lcdStr);

		calibrate_offset();

		//Now calibrate the slope
		for (i = 0; i < NUM_SLOPE_CAL_POINTS; i++)
		{
			load_string(st_load_cal_info, st_calibrate_slope + i, lcdStr);
			lcd_printf(1, 3, lcdStr);
			load_string(st_load_cal_info, st_calibrate_start, lcdStr);
			lcd_printf(1, 4, lcdStr);

			//Wait for them to press - no way out of this apart from power off
			key_pressed = 0;
			timer_restart();

			while ((key_pressed == 0) && !abort)
			{
				abort = calibrate_idle(&key_pressed);
				//show_measured_value();
			}

			if (!abort)
			{
				load_string(st_load_cal_info, st_calibrate_wait, lcdStr);
				lcd_printf(1, 4, lcdStr);

				measure_slope_error(i);
			}
			else
			{
				break;
			}
		}
	}

	if (!abort)
	{
		lcd_clear_line(3);
		load_string(st_load_cal_info, st_calibrate_wait, lcdStr);
		lcd_printf(1, 4, lcdStr);

		calibrate_slope();

		cal_offset[0] = new_cal_offset[0];
		cal_offset[1] = new_cal_offset[1];
		cal_slope[0] = new_cal_slope[0];
		cal_slope[1] = new_cal_slope[1];

		save_calibration_info();
		load_string(st_load_cal_info, st_calibrate_complete, lcdStr);
		lcd_printf(1, 4, lcdStr);
		delay_ms(1000);
		test_state[0] = state_complete;
		test_state[1] = state_complete;
	}
	units = old_units;
	calibrating = false;
}

//----------------------------------------------------------------------------------
bool calibrate_idle(int8 * pressed)
//----------------------------------------------------------------------------------
{
	unsigned int8 command;

	*pressed = 0;
	command = check_command();
	if (command == CMD_GET_MODE)
   	{
		tempStr[0] = CMD_GET_MODE;
		tempStr[1] = state_calibrating;
		rs232_write_bytes(&tempStr[0], 2);
   	}

	if (timer_has_ticked())
	{
		timer_restart();

		if (check_start_key())
		{
			*pressed = 1;
		}
	}

	if (input(ONOFFKEY))
	{
		switch_off();
		return true;
	}
	else
	{
		return false;
	}
}

//----------------------------------------------------------------------------------
void calibrate_offset(void)
//----------------------------------------------------------------------------------
{
	unsigned int16 pre1;
	unsigned int16 pre2;
	unsigned int8 i;

	if (!dual_channel)
	{
		pre1 = 0;

		for (i = 0; i < 10; i++)
		{
			read_single_pressure(&pressure[0]);
			pre1 += pressure[0];
			delay_ms(100);
		}

		pre1 /= 10;
		new_cal_offset[0] = pre1;
	}
	else
	{
		pre1 = 0;
		pre2 = 0;

		for (i = 0; i < 10; i++)
		{
			read_dual_pressures(&pressure[0], &pressure[1]);
			pre1 += pressure[0];
			pre2 += pressure[1];
			delay_ms(100);
		}

		pre1 /= 10;
		new_cal_offset[0] = pre1;

		pre2 /= 10;
		new_cal_offset[1] = pre2;
	}
}

//----------------------------------------------------------------------------------
void measure_slope_error(int8 position)
//----------------------------------------------------------------------------------
{
	unsigned int32 pre1;
	unsigned int32 pre2;
	unsigned int8 i;

	pre1 = 0;
	pre2 = 0;

	if (!dual_channel)
	{
		for (i = 0; i < 10; i++)
		{
			read_single_pressure(&pressure[0]);
			pre1 += pressure[0];
			delay_ms(100);
		}

		pre1 /= 10;
		pre1 -= new_cal_offset[0];
		slope_errors[0][position] = (signed int16)pre1 - slope_points[position];
	}
	else
	{
		for (i = 0; i < 10; i++)
		{
			read_dual_pressures(&pressure[0], &pressure[1]);
			pre1 += pressure[0];
			pre2 += pressure[1];
			delay_ms(100);
		}

		pre1 /= 10;
		pre1 -= new_cal_offset[0];
		slope_errors[0][position] = (signed int16)pre1 - slope_points[position];

		pre2 /= 10;
		pre2 -= new_cal_offset[1];
		slope_errors[1][position] = (signed int16)pre2 - slope_points[position];
	}
}

//----------------------------------------------------------------------------------
void calibrate_slope(void)
//----------------------------------------------------------------------------------
{
	int8 i;
	signed int32 slope0;
	signed int32 slope1;

	slope0 = 0;
	slope1 = 0;

	for (i = 0; i < NUM_SLOPE_CAL_POINTS; i++)
	{
		slope0 += ((100000 * (signed int32)(slope_errors[0][i] + slope_points[i])) / (signed int32)slope_points[i]);
		slope1 += ((100000 * (signed int32)(slope_errors[1][i] + slope_points[i])) / (signed int32)slope_points[i]);
	}

	slope0 /= NUM_SLOPE_CAL_POINTS;
	slope0 *= 1800;
	slope0 /= 100000;

	slope1 /= NUM_SLOPE_CAL_POINTS;
	slope1 *= 1800;
	slope1 /= 100000;

	new_cal_slope[0] = 180000000 / slope0;
	new_cal_slope[1] = 180000000 / slope1;
}

//----------------------------------------------------------------------------------
void save_calibration_info(void)
//----------------------------------------------------------------------------------
{
	//save this to EEPROM
	write_eeprom_data();
}

//----------------------------------------------------------------------------------
void load_calibration_info(void)
//----------------------------------------------------------------------------------
{
	//load this from EEPROM
	read_eeprom_data();
}
// =============================================================
#endif
