//===================================================================
//                                                                  =
// Author:    Martin Berriman                                       =
// Copyright: Innovatronics Ltd 2006                                =
//                                                                  =
//===================================================================

#ifndef _MAINH
#define _MAINH

#nolist

// INCLUDES =========================================================
#include <18F2525.h>
#include "global.h"
// END INCLUDES =====================================================

// FUSES ============================================================
#device *=16
#FUSES NOWDT
#FUSES HS                    	//High speed Osc (> 4mhz)
#FUSES PROTECT             		//Code protected from reading
#FUSES NODEBUG               	//No Debug mode for ICD
#FUSES NOCPD                 	//No EE protection
#FUSES NOWRT                 	//Program memory not write protected
#FUSES NOWRTB                	//Boot block not write protected
#FUSES NOEBTR                	//Memory not protected from table reads
#FUSES NOEBTRB               	//Boot block not protected from table reads
#FUSES NOCPB                 	//No Boot Block code protection
#FUSES BROWNOUT              	//Reset when brownout detected
#FUSES PUT                   	//Power Up Timer
#FUSES STVREN                	//Stack full/underflow will cause reset
#FUSES NOLVP									//Low Voltage Programming on B3(PIC16) or B5(PIC18)
#FUSES NOWRTD                	//Data EEPROM not write protected
#FUSES IESO                  	//Internal External Switch Over mode enabled
#FUSES FCMEN                 	//Fail-safe clock monitor enabled
#FUSES NOPBADEN								//PORTB pins are not configured as analog input channels on RESET
#FUSES NOWRTC                	//configuration not registers write protected
#FUSES MCLR                  	//Master Clear pin enabled
#FUSES NOXINST               	//Extended set extension and Indexed Addressing mode disabled (Legacy mode)
// END FUSES ========================================================

// DEFINES ==========================================================
#define I2C_SDA  PIN_C4
#define I2C_SCL  PIN_C3

//Things that might need changing
//#define HWVer 1
#define HWVer 2 //moved position of sensors and changed to use FT232R instead of FT232BM
#define SWVer 18
#define STARTPRESSTIME	2		//this is in seconds so 2 seconds
#define SPLASHTIME      50	//this is in 10's of a second so 50 = 5 seconds
// END DEFINES ======================================================

// HELPERS ==========================================================
#use delay(clock=20000000)
#use rs232(baud=115200,parity=N,xmit=PIN_C6,rcv=PIN_C7,bits=8,ERRORS)
#use i2c(master, sda = I2C_SDA, scl = I2C_SCL)
#use fast_io(A)
#use fast_io(B)
#use fast_io(C)
// END HELPERS ======================================================

// PROTOTYPES =======================================================
void read_setup(void);
void reset(void);
bool check_start_key(void);
void switch_on(void);
void switch_off(void);
void initialise(void);
void main(void);
// END PROTOTYPES ===================================================

#list

// =============================================================
#endif
