//===================================================================
//                                                                  =
// Author:    Martin Berriman                                       =
// Copyright: Innovatronics Ltd 2006                                =
//                                                                  =
//===================================================================

#ifndef _STRINGSC
#define _STRINGSC

// HEADERS ==========================================================
#include "strings.h"
// END HEADERS ======================================================

// C FILES ==========================================================
// END C FILES ======================================================

// MACROS =========================================================
// END MACROS =====================================================

// GLOBALS ==========================================================
enum st_load_type {st_load_units, st_load_startup, st_load_test_info, st_load_tests_info, st_load_cal_info};

const char gUnitsStrings[20] = {"Pa\x00mbar\x00mmH2O\x00InH2O"}; //NULL separated strings
const int8 gUnitsOffsets[4] = {0,3,8,14}; //offset table for strings
const int8 gUnitsDecimals[4] = {0, 2, 1, 2};
enum st_units {st_units_pa, st_units_mbar, st_units_mmh2o, st_units_inh2o, st_units_max};

const char gStartupStrings[126] = {"     Suitcheck      \x00  Hardware Ver:###  \x00  Software Ver:###  \x00 Serial No.: #####  \x00 Calibrated:####### \x00Ready to begin test "}; //NULL separated strings
const int8 gStartupOffsets[6] = {0,21,42,63,84,105}; //offset table for strings
enum st_startup {st_startup_title, st_startup_ver_hw, st_startup_ver_sw, st_startup_serial, st_startup_cal, st_startup_idle}; //Enum to enable easy access

const char gTestSingleStrings[189] = {"    Inflate Fast    \x00    Inflate Slow    \x00Stabilise: ##m ##s  \x00 Testing:#   of #   \x00 Pressure:#         \x00   Test Complete    \x00    Result:Pass     \x00    Result:Fail     \x00 Preparing to test  "}; //NULL separated strings
const int8 gTestSingleOffsets[9] = {0,21,42,63,84,105,126,147,168}; //offset table for strings
enum st_test_single {st_test_single_inflate_fast, st_test_single_inflate_slow, st_test_single_stabilising, st_test_single_testing, st_test_single_pressure, st_test_single_complete, st_test_single_pass, st_test_single_fail, st_test_single_prep_test};

const char gTestDualStrings[157] = {"     Units:#        \x00 Suit:A    Suit:B   \x00 #         #        \x00 P:#       P:#      \x00InflateF\x00InflateS\x00Stab:#   \x00###of###\x00Complete\x00Passed\x00Failed\x00Prepare\x00Idle"}; //NULL separated strings
const int8 gTestDualOffsets[13] = {0,21,42,63,84,93,102,112,121,130,137,144,152}; //offset table for strings
enum st_test_dual {st_test_dual_units, st_test_dual_suits, st_test_dual_info, st_test_dual_pressure, st_test_dual_inflate_fast, st_test_dual_inflate_slow, st_test_dual_stabilise, st_test_dual_samples, st_test_dual_complete, st_test_dual_passed, st_test_dual_failed, st_test_dual_prep_test, st_test_dual_idle}; //Enum to enable easy access

const char gCalibrateStrings[189] = {"  Calibration Mode  \x00 Set pressure 1800Pa\x00 Set pressure 900Pa \x00 Set pressure 1350Pa\x00 Set pressure 450Pa \x00 Set pressure 0Pa   \x00  then press start  \x00   Please Wait...   \x00Calibration Complete"}; //NULL separated strings
const int8 gCalibrateOffsets[9] = {0,21,42,63,84,105,126,147,168}; //offset table for strings
enum st_calibrate {st_calibrate_mode, st_calibrate_slope, st_calibrate_slope2, st_calibrate_slope3, st_calibrate_slope4, st_calibrate_min, st_calibrate_start, st_calibrate_wait, st_calibrate_complete}; //Enum to enable easy access
// END GLOBALS ======================================================

//----------------------------------------------------------------------------------
void load_string(st_load_type string_type, int8 string_num, char * str)
//----------------------------------------------------------------------------------
{
	int8 offset;
	char data;

	switch (string_type)
	{
		case st_load_units:
			{
				offset = gUnitsOffsets[string_num];
      			#ignore_warnings 201  //assignment within relational test 
				while (data = gUnitsStrings[offset++])
      			{
      				*str++ = data;
      			}
      			#ignore_warnings NONE
			}
			break;
		case st_load_startup:
			{
				offset = gStartupOffsets[string_num];
      			#ignore_warnings 201  //assignment within relational test 
      			while (data = gStartupStrings[offset++])
      			{
      				*str++ = data;
      			}
      			#ignore_warnings NONE
			}
			break;
		case st_load_test_info:
			{
				offset = gTestSingleOffsets[string_num];
      			#ignore_warnings 201  //assignment within relational test 
      			while (data = gTestSingleStrings[offset++])
      			{
      				*str++ = data;
      			}
      			#ignore_warnings NONE
			}
			break;
		case st_load_tests_info:
			{
				offset = gTestDualOffsets[string_num];
      			#ignore_warnings 201  //assignment within relational test 
      			while (data = gTestDualStrings[offset++])
      			{
      				*str++ = data;
      			}
      			#ignore_warnings NONE
			}
			break;
		case st_load_cal_info:
			{
				offset = gCalibrateOffsets[string_num];
      			#ignore_warnings 201  //assignment within relational test 
      			while (data = gCalibrateStrings[offset++])
      			{
      				*str++ = data;
      			}
      			#ignore_warnings NONE
			}
			break;
	}
	*str = '\0';
}

//----------------------------------------------------------------------------------
void load_string_name(char * str)
//----------------------------------------------------------------------------------
{
	load_string(st_load_startup, st_startup_title, str);
}

//----------------------------------------------------------------------------------
void load_string_version_hw(char * str)
//----------------------------------------------------------------------------------
{
	load_string(st_load_startup, st_startup_ver_hw, tempStr);
	fix_string_int(tempStr, HWVer, 3);
	strncpy(str, tempStr, 20);
}

//----------------------------------------------------------------------------------
void load_string_version_sw(char * str)
//----------------------------------------------------------------------------------
{
	load_string(st_load_startup, st_startup_ver_sw, tempStr);
	fix_string_int(tempStr, SWVer, 3);
	strncpy(str, tempStr, 20);
}

//----------------------------------------------------------------------------------
void load_string_serial(char * str, unsigned int16 serial_no)
//----------------------------------------------------------------------------------
{
	load_string(st_load_startup, st_startup_serial, tempStr);
	fix_string_long(tempStr, serial_no, 5, false);
	strncpy(str, tempStr, 20);
}

//----------------------------------------------------------------------------------
void load_string_caldate(char * str, char * cal_date)
//----------------------------------------------------------------------------------
{
	load_string(st_load_startup, st_startup_cal, tempStr);
	fix_string_str(tempStr, cal_date);
	strncpy(str, tempStr, 20);
}

//----------------------------------------------------------------------------------
void load_string_idle(char * str)
//----------------------------------------------------------------------------------
{
	load_string(st_load_startup, st_startup_idle, str);
}

//----------------------------------------------------------------------------------
void load_string_std_name(char * str)
//----------------------------------------------------------------------------------
{
	char name[13];
	char * ptr1;
	char * ptr2;
	int8 len;

	strcpy(name, test_standard_info.name);
	ptr1 = &name[0];

	while (*ptr1 != '#')
	{
		ptr1++;
	}
	*ptr1 = '\0';

	len = strlen(name);

	strcpy(str, "                    ");
	ptr1 = &name[0];
	ptr2 = str;

	ptr2 += 9 - (len / 2); //put it in the middle

	while (*ptr1 != '\0')
	{
		*ptr2++ = *ptr1++;
	}
}

//----------------------------------------------------------------------------------
void load_string_test_info(char * str, st_test_single string_num)
//----------------------------------------------------------------------------------
{
	char s[6];

	load_string(st_load_test_info, string_num, tempStr);
	switch(string_num)
	{
		case st_test_single_pressure:
			fix_string_pressure(tempStr, pressure[0], true);
			load_string(st_load_units, units, s);
			fix_string_str(tempStr, s);
			break;
		case st_test_single_testing:
			fix_string_long(tempStr, sample_num[0], 3, false);
			fix_string_long(tempStr, test_standard_info.total_samples, 3, false);
			break;
	}
	strncpy(str, tempStr, 20);
}

//----------------------------------------------------------------------------------
void load_string_tests_info(char * str, st_test_dual string_num)
//----------------------------------------------------------------------------------
{
	char s[6];

	load_string(st_load_tests_info, string_num, tempStr);
	switch(string_num)
	{
		case st_test_dual_pressure:
			fix_string_pressure(tempStr, pressure[0], true);
			load_string(st_load_units, units, s);
			fix_string_str(tempStr, s);
			break;
	}
	strncpy(str, tempStr, 20);
}

//----------------------------------------------------------------------------------
void fix_string_int(char * str, int8 val, int8 zeros)
//----------------------------------------------------------------------------------
{
	char s[6];
	char * ptr;

	while (*str != '#')
		str++;
//char number[9]; 
//sprintf(number, "%08d", 12); 
//printf(&number[8-n]);
	switch(zeros)
	{
		case 0: sprintf(s, "%u",  val); break;
		case 1: sprintf(s, "%01U", val); break;
		case 2: sprintf(s, "%02U", val); break;
		case 3: sprintf(s, "%03U", val); break;
		case 4: sprintf(s, "%04U", val); break;
		case 5: sprintf(s, "%05U", val); break;
	}

	ptr = s;
	while (*ptr != '\0')
		*str++ = *ptr++;
}

//----------------------------------------------------------------------------------
void fix_string_long(char * str, int16 val, int8 zeros, bool rehash)
//----------------------------------------------------------------------------------
{
	char s[6];
	char * ptr;

	while (*str != '#')
		str++;

	switch(zeros)
	{
		case 0: sprintf(s, "%lu",   val); break;
		case 1: sprintf(s, "%01lu", val); break;
		case 2: sprintf(s, "%02lu", val); break;
		case 3: sprintf(s, "%03lu", val); break;
		case 4: sprintf(s, "%04lu", val); break;
		case 5: sprintf(s, "%05lu", val); break;
	}

	if (rehash)
	{
		ptr = s;
		while (*ptr != '\0')
			ptr++;

		*ptr++ = '#';
		*ptr = '\0';
	}

	ptr = s;
	while (*ptr != '\0')
		*str++ = *ptr++;
}

//----------------------------------------------------------------------------------
void fix_string_pressure(char * str, int16 val, bool rehash)
//----------------------------------------------------------------------------------
{
	char s[8];
	char * ptr;
	int16 multiplier = 1;
	int8 decimal_places;
	int8 zeros;
	unsigned int16 fraction_num = 0;

	decimal_places = gUnitsDecimals[units];
	zeros = decimal_places;

	if (decimal_places > 0)
	{
		while (decimal_places > 0)
		{
			fraction_num += multiplier * (val % 10);
			val /= 10;
			decimal_places--;
			multiplier *= 10;
		}
	
		switch (zeros)
		{
			case 1: sprintf(s, "%lu.%01lu", val, fraction_num); break;
			case 2: sprintf(s, "%lu.%02lu", val, fraction_num); break;
			case 3: sprintf(s, "%lu.%03lu", val, fraction_num); break;
			case 4: sprintf(s, "%lu.%04lu", val, fraction_num); break;
		}
	}
	else
	{
		sprintf(s, "%lu", val);
	}

	while (*str != '#')
		str++;

	if (rehash)
	{
		ptr = s;
		while (*ptr != '\0')
			ptr++;

		*ptr++ = '#';
		*ptr = '\0';
	}

	ptr = s;
	while (*ptr != '\0')
		*str++ = *ptr++;
}

//----------------------------------------------------------------------------------
void fix_string_str(char * str, char * str2)
//----------------------------------------------------------------------------------
{
	while (*str != '#')
		str++;

	while (*str2 != '\0')
		*str++ = *str2++;
}

// =============================================================
#endif
